# *************************************************************************
#
#  Copyright (c) 2025 - Datatailr Inc.
#  All Rights Reserved.
#
#  This file is part of Datatailr and subject to the terms and conditions
#  defined in 'LICENSE.txt'. Unauthorized copying and/or distribution
#  of this file, in parts or full, via any medium is strictly prohibited.
# *************************************************************************

import importlib
import json
import base64


class dt_json:
    @staticmethod
    def dumps(obj):
        d = obj.__dict__ if hasattr(obj, "__dict__") else obj
        d["__class__"] = obj.__class__.__name__
        return json.dumps(d)

    @classmethod
    def loads(cls, json_str):
        d = json.loads(json_str)
        class_name = d.get("__class__")
        module = importlib.import_module("datatailr")
        if class_name and hasattr(module, class_name):
            target_class = getattr(module, class_name)
            obj = target_class.__new__(target_class)
            if hasattr(obj, "from_json"):
                obj.from_json(d)
                return obj
            # fallback: set attributes directly
            for k, v in d.items():
                if k != "__class__":
                    setattr(obj, k, v)
            return obj
        return d  # fallback to dict if not registered

    @classmethod
    def load(cls, json_file):
        return json.load(json_file)


def encode_json(obj) -> str:
    """
    Encode an object to a JSON string with class information.
    """
    json_str = json.dumps(
        obj, default=lambda o: o.__dict__ if hasattr(o, "__dict__") else o
    )
    return base64.b64encode(json_str.encode("utf-8")).decode("utf-8")


def is_base64(s: str) -> bool:
    try:
        # Try decoding the string
        base64.b64decode(s, validate=True)
        return True
    except (ValueError, TypeError):
        # If decoding fails, it's not valid Base64
        return False


def decode_json(encoded_str: str):
    """
    Decode a JSON string back to an object.
    """
    if is_base64(encoded_str):
        json_str = base64.b64decode(encoded_str.encode("utf-8")).decode("utf-8")
    else:
        json_str = encoded_str
    return json.loads(json_str)



##########################################################################
#
#  Copyright (c) 2025 - Datatailr Inc.
#  All Rights Reserved.
#
#  This file is part of Datatailr and subject to the terms and conditions
#  defined in 'LICENSE.txt'. Unauthorized copying and/or distribution
#  of this file, in parts or full, via any medium is strictly prohibited.
##########################################################################

from __future__ import annotations

import os
import tempfile

from datatailr.wrapper import dt__Blob

# Datatailr Blob API Client
__client__ = dt__Blob()
__user__ = os.getenv("USER", "root")


class Blob:
    def ls(self, path: str) -> list[str]:
        """
        List files in the specified path.

        :param path: The path to list files from.
        :return: A list of file names in the specified path.
        """
        return __client__.ls(path)

    def get_file(self, name: str, path: str):
        """
        Copy a blob file to a local file.

        Args:
            name (str): The name of the blob to retrieve.
            path (str): The path to store the blob as a file.
        """
        return __client__.cp(f"blob://{name}", path)

    def put_file(self, name: str, path: str):
        """
        Copy a local file to a blob.

        Args:
            name (str): The name of the blob to create.
            path (str): The path of the local file to copy.
        """
        return __client__.cp(path, f"blob://{name}")

    def exists(self, name: str) -> bool:
        """
        Check if a blob exists.

        Args:
            name (str): The name of the blob to check.

        Returns:
            bool: True if the blob exists, False otherwise.
        """
        return __client__.exists(name)

    def delete(self, name: str):
        """
        Delete a blob.

        Args:
            name (str): The name of the blob to delete.
        """
        return __client__.rm(name)

    def get_blob(self, name: str):
        """
        Get a blob object.

        Args:
            name (str): The name of the blob to retrieve.

        Returns:
            Blob: The blob object.
        """
        # Since direct reading and writting of blobs is not implemented yet, we are using a temporary file.
        # This is a workaround to allow reading the blob content directly from the blob storage.
        temp_dir = f"/home/{__user__}/tmp"
        if not os.path.exists(temp_dir):
            temp_dir = "/tmp"
        else:
            temp_dir += "/.dt"
            os.makedirs(temp_dir, exist_ok=True)
        with tempfile.NamedTemporaryFile(dir=temp_dir, delete=True) as temp_file:
            self.get_file(name, temp_file.name)
            with open(temp_file.name, "r") as f:
                return f.read()

    def put_blob(self, name: str, blob):
        """
        Put a blob object into the blob storage.

        Args:
            name (str): The name of the blob to create.
            blob: The blob object to store.
        """
        # Since direct reading and writting of blobs is not implemented yet, we are using a temporary file.
        # This is a workaround to allow writing the blob content directly to the blob storage.
        with tempfile.NamedTemporaryFile(delete=True) as temp_file:
            if isinstance(blob, bytes):
                with open(temp_file.name, "wb") as f:
                    f.write(blob)
            else:
                with open(temp_file.name, "w") as f:
                    f.write(blob)
            self.put_file(name, temp_file.name)



# *************************************************************************
#
#  Copyright (c) 2025 - Datatailr Inc.
#  All Rights Reserved.
#
#  This file is part of Datatailr and subject to the terms and conditions
#  defined in 'LICENSE.txt'. Unauthorized copying and/or distribution
#  of this file, in parts or full, via any medium is strictly prohibited.
# *************************************************************************

import os

from datatailr.scheduler.constants import BATCH_JOB_ARGUMENTS


def get_available_env_args():
    """
    Get the available environment variables for batch job arguments.

    This function retrieves the environment variables that match the keys defined in DATATAILR_BATCH_JOB_ARGUMENTS.

    Returns:
        dict: A dictionary of available environment variables for batch jobs.
    """
    available_args = {}
    for key, value in os.environ.items():
        arg_key = key.replace("DATATAILR_BATCH_ARG_", "").lower()
        if arg_key in BATCH_JOB_ARGUMENTS:
            available_args[arg_key] = value
    return available_args



##########################################################################
#
#  Copyright (c) 2025 - Datatailr Inc.
#  All Rights Reserved.
#
#  This file is part of Datatailr and subject to the terms and conditions
#  defined in 'LICENSE.txt'. Unauthorized copying and/or distribution
#  of this file, in parts or full, via any medium is strictly prohibited.
##########################################################################


class DatatailrError(Exception):
    """Base class for all DataTailr exceptions."""

    pass


class BatchJobError(DatatailrError):
    """Exception raised for errors related to batch jobs."""

    pass


class ScheduleError(DatatailrError):
    """Exception raised for errors related to scheduling."""

    pass



from typing import Dict, Optional, Union, Callable
from datatailr.build import Image
from datatailr import ACL, Environment, User
from datatailr.scheduler.base import Job, JobType, EntryPoint, Resources


class App(Job):
    def __init__(
        self,
        name: str,
        entrypoint: Callable,
        environment: Optional[Environment] = Environment.DEV,
        image: Optional[Image] = None,
        run_as: Optional[Union[str, User]] = None,
        resources: Resources = Resources(),
        acl: Optional[ACL] = None,
        python_version: str = "3.12",
        python_requirements: str = "",
        build_script_pre: str = "",
        build_script_post: str = "",
        env_vars: Dict[str, str | int | float | bool] = {},
        update_existing: bool = False,
    ):
        entrypoint = EntryPoint(JobType.APP, entrypoint)
        super().__init__(
            name=name,
            type=JobType.APP,
            entrypoint=entrypoint,
            environment=environment,
            image=image,
            run_as=run_as,
            resources=resources,
            acl=acl,
            python_version=python_version,
            python_requirements=python_requirements,
            build_script_pre=build_script_pre,
            build_script_post=build_script_post,
            env_vars=env_vars,
            update_existing=update_existing,
        )


class Service(Job):
    def __init__(
        self,
        name: str,
        entrypoint: Callable,
        environment: Optional[Environment] = Environment.DEV,
        image: Optional[Image] = None,
        run_as: Optional[Union[str, User]] = None,
        resources: Resources = Resources(),
        acl: Optional[ACL] = None,
        python_version: str = "3.12",
        python_requirements: str = "",
        build_script_pre: str = "",
        build_script_post: str = "",
        env_vars: Dict[str, str | int | float | bool] = {},
        update_existing: bool = False,
    ):
        entrypoint = EntryPoint(JobType.SERVICE, entrypoint)
        super().__init__(
            name=name,
            type=JobType.SERVICE,
            entrypoint=entrypoint,
            environment=environment,
            image=image,
            run_as=run_as,
            resources=resources,
            acl=acl,
            python_version=python_version,
            python_requirements=python_requirements,
            build_script_pre=build_script_pre,
            build_script_post=build_script_post,
            env_vars=env_vars,
            update_existing=update_existing,
        )


class ExcelAddin(Job):
    def __init__(
        self,
        name: str,
        entrypoint: Callable,
        environment: Optional[Environment] = Environment.DEV,
        image: Optional[Image] = None,
        run_as: Optional[Union[str, User]] = None,
        resources: Resources = Resources(),
        acl: Optional[ACL] = None,
        python_version: str = "3.12",
        python_requirements: str = "",
        build_script_pre: str = "",
        build_script_post: str = "",
        env_vars: Dict[str, str | int | float | bool] = {},
        update_existing: bool = False,
    ):
        entrypoint = EntryPoint(JobType.EXCEL, entrypoint)
        super().__init__(
            name=name,
            type=JobType.EXCEL,
            entrypoint=entrypoint,
            environment=environment,
            image=image,
            run_as=run_as,
            resources=resources,
            acl=acl,
            python_version=python_version,
            python_requirements=python_requirements,
            build_script_pre=build_script_pre,
            build_script_post=build_script_post,
            env_vars=env_vars,
            update_existing=update_existing,
        )
